/*
 * Copyright 2017 by Kappich Systemberatung Aachen
 *
 * This file is part of de.kappich.pat.gnd.
 *
 * de.kappich.pat.gnd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.kappich.pat.gnd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.kappich.pat.gnd.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.kappich.pat.gnd.properties;

import de.bsvrz.sys.funclib.kappich.annotations.Nullable;
import de.kappich.pat.gnd.utils.SpringUtilities;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import java.awt.Dimension;

/**
 * Diese Klasse verkapselt die Eigenschaft Durchmesser.
 *
 * @author Kappich Systemberatung
 * @version $Revision$
 */
public class DiameterProperty extends AbstractProperty {
	private static final DiameterProperty _instance = new DiameterProperty();
	private static final String KEY = "diameter";
	private static final Double DEFAULT_DIAMETER = 0.;

	private DiameterProperty() {
		super("Durchmesser");
	}

	public static Property getInstance() {
		return _instance;
	}

	@Override
	public Object getDefaultValue() {
		return DEFAULT_DIAMETER;
	}

	@Override
	public String getKey() {
		return KEY;
	}

	@Override
	public PropertyPanel getPropertyPanel(@Nullable Object value, boolean changeable) {
		if ( value!=null) {
			return new DiameterPanel((Double) value, changeable);
		} else {
			return new DiameterPanel(DEFAULT_DIAMETER, changeable);
		}
	}

	private static class DiameterPanel extends JPanel implements PropertyPanel {
		private final JSpinner _diameterSpinner = new JSpinner(new SpinnerNumberModel(0, 0, 10000, 1));

		public DiameterPanel(Double diameter, boolean changeable) {
			JLabel label = new JLabel("Durchmesser: ");
			_diameterSpinner.setMaximumSize(new Dimension(200, 20));
			_diameterSpinner.setValue(diameter);
			_diameterSpinner.setEnabled(changeable);
			setLayout(new SpringLayout());
			add(label);
			add(_diameterSpinner);
			SpringUtilities.makeCompactGrid(this, 2, 5, 5);
		}

		@Override
		public void addListener(final PropertyPanelListener listener) {
			ChangeListener changeListener = e -> listener.stateChanged();
			_diameterSpinner.addChangeListener(changeListener);
		}

		@Override
		public Object getPropertyValue() {
			return _diameterSpinner.getValue();
		}

		@Override
		public void setSelectionEnabled(final boolean enabled) {
			_diameterSpinner.setEnabled(enabled);
		}
	}
}
